/*******************************************************************************
* FILE NAME: auto_arm_routines.c
*
*	Version 4.0	last updated 8/2/04::
*
* DESCRIPTION:
*  Autonomous arm functionse
*
* USAGE:
*  Just use it...
*
*******************************************************************************/

#ifdef _DEBUG
#include "print_routines.h"
#endif

#include "ifi_aliases.h"
#include "ifi_default.h"
#include "ifi_utilities.h"

#include "motor_routines.h"
#include "auto_routines.h"
#include "user_routines.h"
#include "clock_routines.h"
#include "sensor_routines.h"


/*******************************************************************************
* FUNCTION NAME: Arm_Open
* PURPOSE:       Simple open arms function for function-based autonomous mode.
*******************************************************************************/
void Arm_Open(void)
{
if (lastDriveCmd != ARMOPN)
{
#ifdef _DEBUG
    PrintString((char *) " **Close_Arms** ");
    PrintCR();
#endif
} 

  TETRA_GRABBER = 1;
  TETRA_GRABBER_REV = 0;  // Not necessary only here as a future reminder
  lastArmCmd = ENDCMD;  // In case there are two of the same type cmd in a row
  Arm_Status = COMPLETE;

}

/*******************************************************************************
* FUNCTION NAME: Arm_Close
* PURPOSE:       Simple close arms function for function-based autonomous mode.
*******************************************************************************/
void Arm_Close(void)
{
  if (lastDriveCmd != ARMCLS)
  {
#ifdef _DEBUG
    PrintString((char *) " **Close_Arms** ");
    PrintCR();
#endif
  } 
  TETRA_GRABBER = 0;
  TETRA_GRABBER_REV = 0; // Not necessary as long as it's been initialized to 0 somewhere
  lastArmCmd = ENDCMD;  // In case there are two of the same type cmd in a row
  Arm_Status = COMPLETE;
}

/*******************************************************************************
* FUNCTION NAME: Arm_Forward
* PURPOSE:       Simple timed arm forward function for autonomous mode.
*
* Notes:         arms down in starting position
*                ?=10pt ball height
*                ?=mobile goal 2x ball height
*                ?=stationary goal 2x ball height
*                ?=straight up
*******************************************************************************/
void Arm_Forward_Time(int time, int speed)
{
  static unsigned long duration;
  int arm_pot;
 
  if (lastArmCmd != ARMFWD)
  {
    lastArmCmd = ARMFWD;
    duration = Clockds + time; // Current time + duration = cmd completed
    holdarm = FALSE;

#ifdef _DEBUG
    PrintString((char *) " **Arms_Forward** ");
    PrintCR();
#endif
  }

  if (duration > Clockds)
  { // Keep going
	ARM_PWM = speed;
    Arm_Status = INCOMPLETE;
  }
  else
  { // All done (move on to next drive command)
	ARM_PWM = NEUTRAL;
    arm_pot = (int) Get_Analog_Value(ARM_POT); //MLM
    holdarm = TRUE;
    Hold_Arm_Position(TRUE, arm_pot, &ARM_PWM); // Initialize arm hold position
    Arm_Status = COMPLETE;
    lastArmCmd = ENDCMD;  // In case there are two of the same type cmd in a row
  }
}


/*******************************************************************************
* FUNCTION NAME: Arm_Backward
* PURPOSE:       Simple timed arm back function for autonomous mode.
*
* Notes:         arms will be coming from any position
*******************************************************************************/
void Arm_Backward_Time(int time, int speed)
{
  static unsigned long duration;
  int arm_pot;

 
  if (lastArmCmd != ARMBCK)
  {
    lastArmCmd = ARMBCK;
    duration = Clockds + time; // Current time + duration = cmd completed
    holdarm = FALSE;

#ifdef _DEBUG
    PrintString((char *) " **Arms_Backward** ");
    PrintCR();
#endif
  }

  if (duration > Clockds)
  { // Keep going
	ARM_PWM = -speed;
    Arm_Status = INCOMPLETE;
  }
  else
  { // All done (move on to next drive command)
	ARM_PWM = -5;
    arm_pot = (int) Get_Analog_Value(ARM_POT); //MLM
    holdarm = TRUE;
    Hold_Arm_Position(TRUE, arm_pot, &ARM_PWM); // Initialize arm hold position
    Arm_Status = COMPLETE;
    lastArmCmd = ENDCMD;  // In case there are two of the same type cmd in a row
  }
}


/*******************************************************************************
* FUNCTION NAME: Arm_Pivot
* PURPOSE:       Position arms to the selected banner encoder setting
* ARGUMENTS:     setting - count of banner encoder stripes from the start position.
* RETURNS:       Status - COMPLETE if function has finished requested task
*                         INCOMPLETE if end conditions have not yet been satisfied
*    negative is up, positive is down 
* Notes: settings - 0=arms down in starting position
*                   ?=10pt ball height
*                   ?=mobile goal 2x ball height
*                   ?=stationary goal 2x ball height
*                   ?=straight up
*******************************************************************************/
void Arm_Pivot(int setting)
{
  unsigned int arm_pot;
  static char last_direction;

  if (lastArmCmd != ARMPVT)
  {
    lastArmCmd = ARMPVT;
    Arm_Status = INCOMPLETE;

    holdarm = FALSE;
    arm_pot = Get_Analog_Value(ARM_POT);
    if (setting < arm_pot) //Stop if arm overshoots
      last_direction = 1; //need to go up
    else if (setting > arm_pot)
      last_direction = 2; // need to go down

#ifdef _DEBUG
    PrintString((char *) " **Arm_Pivot** ");
    PrintCR();
#endif
  } 
   /* negative is up, positive is down */

  arm_pot = Get_Analog_Value(ARM_POT);

  if ((setting < arm_pot) && (last_direction == 1))  // Up
  { 
	    ARM_PWM = -FULLSPEED;
  }
  else if ((setting > arm_pot) && (last_direction == 2)) // Down
  {
	    ARM_PWM = FULLSPEED;
  }
  else
  {
	ARM_PWM = ARMNEUTRAL;
    holdarm = TRUE;
    Hold_Arm_Position(TRUE, setting, &ARM_PWM); //Initialize hold arm position steady
    lastArmCmd = ENDCMD;  // In case there are two of the same type cmd in a row
    Arm_Status = COMPLETE;
  }
}



/*******************************************************************************
* FUNCTION NAME: Arm_Telescope
* PURPOSE:       Position arms to the selected banner encoder setting
* ARGUMENTS:     setting - count of banner encoder stripes from the start position.
* RETURNS:       Status - COMPLETE if function has finished requested task
*                         INCOMPLETE if end conditions have not yet been satisfied
*    negative is up, positive is down 
* Notes: settings - 0=arms down in starting position
*                   ?=10pt ball height
*                   ?=mobile goal 2x ball height
*                   ?=stationary goal 2x ball height
*                   ?=straight up
*******************************************************************************/
void Arm_Telescope(int setting)
{
  unsigned int tele_pot;
  static char last_direction;

  if (lastArmCmd != ARMTEL)
  {
    lastArmCmd = ARMTEL;
    Arm_Status = INCOMPLETE;

    tele_pot = Get_Analog_Value(TELESCOPE_POT);
    if (setting < tele_pot) //Stop if arm overshoots
      last_direction = 1;
    else if (setting > tele_pot)
      last_direction = 2;

#ifdef _DEBUG
    PrintString((char *) " **Arm_Telescope** ");
    PrintCR();
#endif
  } 

  tele_pot = Get_Analog_Value(TELESCOPE_POT);

  if ((setting < tele_pot) && (last_direction == 1))  // In
  { 
	    TELESCOPE_PWM1 = -FULLSPEED;
  }
  else if ((setting > tele_pot) && (last_direction == 2)) // Out
  {
	    TELESCOPE_PWM1 = FULLSPEED;
  }
  else
  {
	TELESCOPE_PWM1 = NEUTRAL;
    lastArmCmd = ENDCMD;  // In case there are two of the same type cmd in a row
    Arm_Status = COMPLETE;
  }
}
