/*******************************************************************************
* FILE NAME: auto_diagnostic_routines.c
*
* DESCRIPTION:
*    Autonomous functions used for diagnostic purposes.
*
* USAGE:
*
*******************************************************************************/

#ifdef _DEBUG
#include "print_routines.h"
#endif

#include "ifi_aliases.h"
#include "ifi_default.h"
#include "ifi_utilities.h"

#include "motor_routines.h"
#include "auto_routines.h"
#include "user_routines.h"
#include "clock_routines.h"
#include "sensor_routines.h"


#ifdef _DIAGNOSTICS_
/*******************************************************************************
* FUNCTION NAME: BalanceF
* PURPOSE:       Diagnostic to measure the values needed to balance the two sides
*                of the drivetrain while driving forward.
*******************************************************************************/
void BalanceF(int dist, int speed)
{
  static int RAdjust=0, LAdjust=0;
  static unsigned long LastClock=0;
  static signed long desired_distance; // in encoder counts
  signed long encoder_dist;
  static char reverseFlag1=TRUE, reverseFlag2=TRUE;

  if (lastDriveCmd != FBALANCE)
  {
    lastDriveCmd = FBALANCE;
#ifdef _DEBUG
    PrintCR();
    PrintString((char *) " **FBALANCE** ");
    PrintCR();
#endif

    /* Disable interrupts while we zero the shaft encoders */
    INTCONbits.GIEL = 0;
    Right_Encoder_Count = 0;
    Left_Encoder_Count = 0;
    INTCONbits.GIEL = 1;
    
    desired_distance = -dist;
    /* Encoder has 256 counts per revolution and travels 16.28 inches per revolution */
    desired_distance = desired_distance * (long)TICKS_PER_REV / (long)ENCODER_INCHES; // Convert from inches to encoder ticks
  }

  encoder_dist = (Right_Encoder_Count + Left_Encoder_Count) / 2;

  if (LastClock != Clockds)
  {
#ifdef _DEBUG
    PrintSignedDecimal(dist); PrintString((char *) "in.");
    PrintString((char *) ", Ticks="); PrintSignedDecimal(desired_distance);
    PrintString((char *) ", Right="); PrintSignedDecimal(Right_Encoder_Count);
    PrintString((char *) ", Left="); PrintSignedDecimal(Left_Encoder_Count);
    PrintString((char *) ", avg="); PrintSignedDecimal(encoder_dist);
    PrintString((char *) ", RAdjust="); PrintSignedDecimal(RAdjust);
    PrintString((char *) ", LAdjust="); PrintSignedDecimal(LAdjust);
    PrintCR();
#endif
  }

  if (desired_distance < encoder_dist)
  {
    Drive_Status = INCOMPLETE;
    WHL_L1 = -speed;
    WHL_R1 = WHL_L1;

    /* Balance the motors */
    WHL_L1 += PWMBalance(BalForLeft, speed) + LAdjust;
//    WHL_R1 += PWMBalance(BalForRight, speed) + RAdjust;
    if (WHL_R1 > 0) WHL_R1 = 0;
    if (WHL_L1 > 0) WHL_L1 = 0;

    if (LastClock != Clockds)
    {
      LastClock = Clockds;
#ifdef _DEBUG
      PrintString((char *) ", WHL_R="); PrintSignedDecimal(WHL_R1);
      PrintString((char *) ", WHL_L="); PrintSignedDecimal(WHL_L1);
      PrintCR();
#endif
  }
  else
  {
    WHL_L1 = NEUTRAL;
    WHL_R1 = NEUTRAL;

    if (Left_Encoder_Count < (Right_Encoder_Count-2))
    {
      if (reverseFlag2)
        LAdjust += 1;
      else
      {
        LAdjust += 5;
        reverseFlag1 = TRUE;
      }
    }
    else if (Left_Encoder_Count > (Right_Encoder_Count+2))
    {
      if (reverseFlag1)
        LAdjust -= 1;
      else
      {
        LAdjust -= 5;
        reverseFlag2 = TRUE;
      }
    }
    lastDriveCmd = ENDCMD;  // In case there are two of the same type cmd in a row
    Drive_Status = COMPLETE;
#ifdef _DEBUG
    PrintString((char *) " END");
    PrintString((char *) ", Ticks="); PrintSignedDecimal(desired_distance);
    PrintString((char *) ", Right="); PrintSignedDecimal(Right_Encoder_Count);
    PrintString((char *) ", Left="); PrintSignedDecimal(Left_Encoder_Count);
    PrintString((char *) ", avg="); PrintSignedDecimal(encoder_dist);
    PrintString((char *) ", Clock="); PrintDecimal(Clockds);
    PrintString((char *) ", RAdjust="); PrintSignedDecimal(RAdjust);
    PrintString((char *) ", LAdjust="); PrintSignedDecimal(LAdjust);
    PrintString((char *) ", WHL_R="); PrintSignedDecimal(WHL_R1);
    PrintString((char *) ", WHL_L="); PrintSignedDecimal(WHL_L1);
    PrintCR();
#endif
  }
}


/*******************************************************************************
* FUNCTION NAME: BalanceR
* PURPOSE:       Diagnostic to measure the values needed to balance the two sides
*                of the drivetrain while driving backward.
* NOTES: Repeatedly run the robot forward & backward at a fixed PWM  w/o resetting
*        Reset to start balancing at a new PWM value.
*******************************************************************************/

void BalanceR(int dist, int speed)
{
  static int RAdjust=0, LAdjust=0;
  static unsigned long LastClock=0;
  static signed long desired_distance; // in encoder counts
  signed long encoder_dist;
  static char reverseFlag1=TRUE, reverseFlag2=TRUE;

  if (lastDriveCmd != RBALANCE)
  {
    lastDriveCmd = RBALANCE;
#ifdef _DEBUG
    PrintCR();
    PrintString((char *) " **RBALANCE** ");
    PrintCR();
#endif

    /* Disable interrupts while we zero the shaft encoders */
    INTCONbits.GIEL = 0;
    Right_Encoder_Count = 0;
    Left_Encoder_Count = 0;
    INTCONbits.GIEL = 1;
    
    desired_distance = dist;
    /* Encoder has 256 counts per revolution and travels 11.05 inches per revolution */
    desired_distance = desired_distance * (long)TICKS_PER_REV / (long)ENCODER_INCHES; // Convert from inches to encoder ticks
  }

  encoder_dist = (Right_Encoder_Count + Left_Encoder_Count) / 2;
    if (LastClock != Clockds)
    {
#ifdef _DEBUG
  PrintSignedDecimal(dist); PrintString((char *) "in.");
  PrintString((char *) ", Ticks="); PrintSignedDecimal(desired_distance);
  PrintString((char *) ", Right="); PrintSignedDecimal(Right_Encoder_Count);
  PrintString((char *) ", Left="); PrintSignedDecimal(Left_Encoder_Count);
  PrintString((char *) ", avg="); PrintSignedDecimal(encoder_dist);
  PrintString((char *) ", Clock="); PrintDecimal(Clockds);
  PrintString((char *) ", RAdjust="); PrintSignedDecimal(RAdjust);
  PrintString((char *) ", LAdjust="); PrintSignedDecimal(LAdjust);
  PrintCR();
#endif
  }

  if (desired_distance > encoder_dist)
  {
    Drive_Status = INCOMPLETE;
    WHL_L1 = speed;
    WHL_R1 = WHL_L1;

    /* Balance the motors */
//    WHL_L1 -= (PWMBalance(BalBckLeft, speed) + LAdjust);
    WHL_R1 -= (PWMBalance(BalBckRight, speed)-RAdjust);
    if (WHL_R1 < 0) WHL_R1 = 0;
    if (WHL_L1 < 0) WHL_L1 = 0;

    if (LastClock != Clockds)
    {
      LastClock = Clockds;
#ifdef _DEBUG
      PrintString((char *) ", WHL_R="); PrintSignedDecimal(WHL_R1);
      PrintString((char *) ", WHL_L="); PrintSignedDecimal(WHL_L1);
      PrintCR();
#endif
    }
  }
  else
  {
    WHL_L1 = NEUTRAL;
    WHL_R1 = NEUTRAL;
    if (Right_Encoder_Count < (Left_Encoder_Count-2))
    {
      if (reverseFlag1)
        RAdjust += 1;
      else
      {
        RAdjust += 5;
        reverseFlag2 = TRUE;
      }
    }
    else if (Right_Encoder_Count > (Left_Encoder_Count+2))
    {
      if (reverseFlag2)
        RAdjust -= 1;
      else
      {
        RAdjust -= 5;
        reverseFlag1 = TRUE;
      }
    }

    lastDriveCmd = ENDCMD;  // In case there are two of the same type cmd in a row
    Drive_Status = COMPLETE;
#ifdef _DEBUG
    PrintString((char *) " END");
    PrintString((char *) ", Right="); PrintSignedDecimal(Right_Encoder_Count);
    PrintString((char *) ", Left="); PrintSignedDecimal(Left_Encoder_Count);
    PrintString((char *) ", RAdjust="); PrintSignedDecimal(RAdjust);
    PrintString((char *) ", LAdjust="); PrintSignedDecimal(LAdjust);
    PrintString((char *) ", WHL_R="); PrintSignedDecimal(WHL_R1);
    PrintString((char *) ", WHL_L="); PrintSignedDecimal(WHL_L1);
    PrintCR();
#endif
  }
}

#else
    void BalanceF(int dist, int speed)
    {
#ifdef _DEBUG
      PrintCR();
      PrintString((char *) "***** BalanceF has not been compiled *****");
      PrintCR();
#endif
    }
    void BalanceR(int dist, int speed)
    {
#ifdef _DEBUG
      PrintCR();
      PrintString((char *) "***** BalanceR has not been compiled *****");
      PrintCR();
#endif
    }
#endif // (_DIAGNOSTICS_)

/*******************************************************************************
* FUNCTION NAME: Balance
* PURPOSE:       Diagnostic to measure the values needed to balance the two sides
*                of the drivetrain while driving forward & backward.
* NOTES: Repeatedly run the robot forward & backward w/o resetting
*        Reset to start balancing at a new PWM value.
*        Might make this a PID balancing process.
*        Eliminate first n distance of a run so we're not testing rampup
*        and we can detect the faster side of the drivetrain
*
*        speed = 0 might indicate step through all speeds (by 10s)
*
*******************************************************************************/
#define STEPSTART 10
#define WINDOW    2
#define NEITHERSIDE 0
#define FORWARD 0
#define BACKWARD 1
#define COAST 0
void Balance(int dist, int speed, int fasterSide)
{
  static int Adjust[2]= {0,0};
  static unsigned long LastClock=0;
  static signed long desired_distance, last_encoder_dist; // in encoder counts
  signed long encoder_dist;
  static char stepSize, direction, DrivingDirection, lastDrivingDirection;
  static char finished[2]={FALSE, FALSE};
  static char State=FORWARD;

  if (lastDriveCmd != BALANCE)
  {
    lastDriveCmd = BALANCE;
#ifdef _DEBUG
    PrintCR();
    PrintString((char *) " **BALANCE** ");
    PrintCR();
#endif
    /* Disable interrupts while we zero the shaft encoders */
    INTCONbits.GIEL = 0;
    Right_Encoder_Count = 0;
    Left_Encoder_Count = 0;
    INTCONbits.GIEL = 1;
    
    desired_distance = dist;
    /* Encoder has 256 counts per revolution and travels 11.05 inches per revolution */
    desired_distance = desired_distance * (long)TICKS_PER_REV / (long)ENCODER_INCHES; // Convert from inches to encoder ticks

    stepSize = STEPSTART;
    direction = NEITHERSIDE; // Which side is faster
    lastDrivingDirection = DrivingDirection = FORWARD;
  }

  encoder_dist = (Right_Encoder_Count + Left_Encoder_Count) / 2;

  if (LastClock != Clockds)
  {
#ifdef _DEBUG
    PrintSignedDecimal(dist); PrintString((char *) "in.");
    PrintString((char *) ", Ticks="); PrintSignedDecimal(desired_distance);
    PrintString((char *) ", Right="); PrintSignedDecimal(Right_Encoder_Count);
    PrintString((char *) ", Left="); PrintSignedDecimal(Left_Encoder_Count);
    PrintString((char *) ", avg="); PrintSignedDecimal(encoder_dist);
#endif
  }

  /* Coast to a stop STATE before reversing direction */
  if (DrivingDirection == COAST)
  {
    if (last_encoder_dist == encoder_dist)
    {
      if (lastDrivingDirection == FORWARD)
        DrivingDirection = BACKWARD;
      else
        DrivingDirection = FORWARD;
      lastDrivingDirection = DrivingDirection;
    }
    else
      last_encoder_dist = encoder_dist;
  }
  /* Driving STATE */
  else if (desired_distance > encoder_dist)
  {
    Drive_Status = INCOMPLETE;
    WHL_R1 = WHL_L1 = speed;

    /* Balance the motors */
    /* It can be either the Left or Right side that's slower */
    if (DrivingDirection == BACKWARD) // Currently driving backward
    {
      if (fasterSide == RIGHTSIDE)
        WHL_R1 -= Adjust[DrivingDirection];
      else
        WHL_L1 -= Adjust[DrivingDirection];

      if (WHL_R1 < 0)   WHL_R1 = 0; // Any of these pretty much means there is no solution, fasterSide is incorrect
      if (WHL_L1 < 0)   WHL_L1 = 0;
      if (WHL_R1 > 127) WHL_R1 = 127;
      if (WHL_L1 > 127) WHL_L1 = 127;
    }
    else // Currently driving forward
    {
      if (fasterSide == RIGHTSIDE)
        WHL_R1 += Adjust[DrivingDirection];
      else
        WHL_L1 += Adjust[DrivingDirection];

      if (WHL_R1 > 0)   WHL_R1 = 0; // Any of these pretty much means there is no solution, fasterSide is incorrect
      if (WHL_L1 > 0)   WHL_L1 = 0;
      if (WHL_R1 < -127) WHL_R1 = -127;
      if (WHL_L1 < -127) WHL_L1 = -127;
    }

    if (LastClock != Clockds)
    {
      LastClock = Clockds;
#ifdef _DEBUG
      PrintString((char *) ", WHL_R=");    PrintSignedDecimal(WHL_R1);
      PrintString((char *) ", WHL_L=");    PrintSignedDecimal(WHL_L1);
      PrintString((char *) ", stepSize="); PrintSignedDecimal(stepSize);
      PrintString((char *) ", Adjust=");   PrintSignedDecimal(Adjust[DrivingDirection]);
      PrintCR();
#endif
    }
  }
  else  // Completed a run in one direction, need to stop & reverse direction
  {
    WHL_L1 = NEUTRAL;
    WHL_R1 = NEUTRAL;

    /* Evaluate the current results */
    if ( (Right_Encoder_Count > (Left_Encoder_Count-WINDOW)) & (Right_Encoder_Count < (Left_Encoder_Count+WINDOW)) )
    { // We are balanced, remember these values, we could do Monte Carlo runs with these values
      /* Store in EEPROM */

      /* Coast to a halt first */
      DrivingDirection = COAST;
      last_encoder_dist = encoder_dist; // Use as the measurement of STOP     

      finished[DrivingDirection] = TRUE;

      if (finished[FORWARD] & finished[BACKWARD])
      {  // We have completely finished
        lastDriveCmd = ENDCMD;
        Drive_Status = COMPLETE;
      }
#ifdef _DEBUG
      PrintString((char *) " END: ");
      PrintString((char *) ", Right="); PrintSignedDecimal(Right_Encoder_Count);
      PrintString((char *) ", Left=");  PrintSignedDecimal(Left_Encoder_Count);
      PrintString((char *) ", Adjust="); PrintSignedDecimal(Adjust[DrivingDirection]);
      PrintCR();
#endif
    }
    /* Not balanced so Adjust motors  */
    else if (Right_Encoder_Count < (Left_Encoder_Count-WINDOW))
    {
      Adjust[DrivingDirection] += stepSize;
      if (direction != LEFTSIDE)
      {
        direction = LEFTSIDE;
        stepSize = stepSize / 2; // halve the step value each time we cross the balance point
        if (stepSize < 1)
          stepSize = 1;
      }
    }
    else // Right_Encoder_Count > (Left_Encoder_Count+WINDOW)
    {
      Adjust[DrivingDirection] -= stepSize;
      if (direction != LEFTSIDE)
      {
        direction = LEFTSIDE;
        stepSize = stepSize / 2; // halve the step value each time we cross the balance point
        if (stepSize < 1)
          stepSize = 1;
      }
    }
  }
}
