/*******************************************************************************
* FILE NAME: print_routines.c
*
* DESCRIPTION:
*  This file contains debug print routines.  
*
* USAGE:
*      define _NO_PRINTS to bypass all print statements
*******************************************************************************/

#include <string.h>

#include "ifi_utilities.h"
#include "ifi_picdefs.h"

#include "print_routines.h"


/******* Print utilities *******/
/*
*      The printf supplied isn't efficient, isn't comprehensive, and has bugs that
*           will display incorrect values. Leaving it out also saves 4300 bytes
*           or 14% of the total program memory available to the user.
*
*      PrintString - prints fixed, unchanging strings 80 byte max (rom) (ifi_utilities)
*      PrintRamString - prints strings constructed on-the-fly (ram) -no length limit
*      PrintCR - print a carrage return to start a new line
*      PrintDecimal - print an unsigned number of any type (doesn't require type-cast)
*      PrintSignedDecimal - print a signed number of any type (doesn't require type-cast)
*
*    Alo available from ifi_utilities: (modified to use _NO_PRINTS)
*      PrintByte - prints byte value in hex (modified to put "0x" in front)
*      PrintWord - prints word value in hex (modified to put "0x" in front)
*      DisplayBufr - print a Hexidecimal dump of the Master uP input or output packet
*                    (modified to put "0x" in front)
*      PacketNum_Check - print the current radio packet number
*
*   Example usage:
*		PrintString((char *) "gyroSum = ");
*       PrintSignedDecimal(gyroSum);
*       PrintString((char *) ", heading(degrees) = ");
*       PrintDecimal(heading);
*       PrintCR();
*
*       sample output:
*            gyroSum = 8734, heading(degrees) = 89
*
*     In the IFI_Loader debug window there are 38 lines each 76 characters wide.
*/

  extern char ifi_printfBufr[]; /* Declared in ifi_utilities.c - Limited to 80 bytes */
                                /* Handy to use since it already exists for this purpose */


/*******************************************************************************
* FUNCTION NAME: PrintRamString
* PURPOSE:       Print a string to the program port.
*                (IFI_Loader debug window or HyperTerminal)
*                Similar to PrintString, but accepts a data (ram) memory string.
* Sample Call:     char buf[20]="Hello World";
*                  PrintRamString(buf);
* CALLED FROM:   anywhere
* ARGUMENTS:     
*     Input: bufr - pointer to data memory string.
* RETURNS:       void
*******************************************************************************/
void PrintRamString(char *bufr)
{
#ifdef _DEBUG

  int len,i;
    
  len = (int) strlen((const char *)bufr);

  for (i=0;i<len;i++)
  {
    TXREG = bufr[i];
    Wait4TXEmpty();
  }
#endif /* _NO_PRINTS */
}

/*******************************************************************************
* FUNCTION NAME: PrintCR
* PURPOSE:       Print a carriage return to the Program Port.
* Sample Call:   PrintCR();
* CALLED FROM:   anywhere
* ARGUMENTS:     none
* RETURNS:       void
*******************************************************************************/
void PrintCR(void)
{
#ifdef _DEBUG

    TXREG = 13;
    Wait4TXEmpty();

#endif /* _NO_PRINTS */
}


/*******************************************************************************
* FUNCTION NAME: PrintDecimal
* PURPOSE:       Print decimal data to the Program Port.
* Sample Call:   PrintDecimal(seconds);
*                For any type unsigned char,int,or long
* CALLED FROM:   anywhere
* ARGUMENTS:     none
*     Input: odata - unsigned char,int, or long to be printed.
* RETURNS:       void
*******************************************************************************/
void PrintDecimal(unsigned long odata)
{
#ifndef _NO_PRINTS

  ultoa(odata, ifi_printfBufr); // Convert to text string
  PrintRamString(ifi_printfBufr);

#endif /* _NO_PRINTS */
}

/*******************************************************************************
* FUNCTION NAME: PrintSignedDecimal
* PURPOSE:       Print signed decimal data to the Program Port.
* Sample Call:   PrintSignedDecimal(encoder);
*                For any type unsigned char,int,or long
* CALLED FROM:   anywhere
* ARGUMENTS:     none
*     Input:odata - signed char,int, or long to be printed.
* RETURNS:       void
*******************************************************************************/
void PrintSignedDecimal(long odata)
{
#ifndef _NO_PRINTS

  ltoa(odata, ifi_printfBufr); // Convert to text string
  PrintRamString(ifi_printfBufr);

#endif /* _NO_PRINTS */
}

