/*******************************************************************************
* FILE NAME: sensor_routines.c
*
* DESCRIPTION:
*  This file contains sensor related routines.
*
* USAGE:
*
*******************************************************************************/

#ifdef _DEBUG
#include "print_routines.h"
#endif

#include "ifi_utilities.h"
#include "ifi_aliases.h"
#include "ifi_default.h"

#include "user_routines.h"
#include "auto_routines.h"
#include "motor_routines.h"
#include "clock_routines.h"


#define VARIABLE_DECLARE  /* causes certain global variables to be declared only once */
  #include "sensor_routines.h"
#undef VARIABLE_DECLARE

#define END_STARTUP 30   // (25)Sample gyro over n cycles to obtain accurate GyroNeutral
#define SETTLE_STARTUP 5  // Allow the gyro to settle down for n cycles after power-up
#define SAMPLE_RATE 40/250   // Sample rate of the gyro - gyro provides samples at 40Hz, we check at 250Hz

/*******************************************************************************
* FUNCTION NAME: Gyro
* PURPOSE:       Initializes and accumulates the gyro
* CALLED FROM:   user_routines_fast.c
* NOTES:
*        The gyro has a range of 1007 to 3 and a neutral point of 519-521
*        It's drift is no better than 300 degrees/hour or 5 degrees/minute
*        and is corrupted if the turn rate exceeds it's limit.
*        The 2004 robot has a maximum turn rate of ~140 degrees/sec which
*        is too close to the max. rating on the ADX150. Should use the ADX300
*        The gyro requires .035 sec to turn on and reach operational spec
*******************************************************************************/
void Gyro()
{
  static long LastGyro;
  long CurrentGyro;
  static unsigned long LastClocksec=0, LastClockms=0;
  static unsigned int GyroStartup=0;
  static long GyroNeutral=0;


  if (GyroTicks > 0)
  {
    /** Update gyro **/
    INTCONbits.GIEL = 0;	/* Disable Low Priority Interrupts */
    GyroTicks--;		/* Avoid the problem of decrementing it during an interrupt */
    INTCONbits.GIEL = 1;	/* Enable Low Priority Interrupts */

    if (GyroStartup == END_STARTUP)
    {
      CurrentGyro = (long)Get_Analog_Value(rc_ana_in01) - GyroNeutral;
      if (min_gyro > CurrentGyro) min_gyro = CurrentGyro;
      if (max_gyro < CurrentGyro) max_gyro = CurrentGyro;

      CurrentGyro = CurrentGyro * SAMPLE_RATE; // Add decimal points of accuracy
      GyroSum += (CurrentGyro + LastGyro) / 2;  // Is this helping or hurting?
      LastGyro = CurrentGyro;
    }
    else /** Initializing Gyro **/
    {
      GyroStartup++;
      /** Sample over n cycles to obtain accurate GyroNeutral **/
      /** Allow n cycles for the gyro to be properly powered-up and settle down **/
      if (GyroStartup > SETTLE_STARTUP)
      {
        GyroNeutral += Get_Analog_Value(rc_ana_in01)<<6; // Increase accuracy
        if (GyroStartup == END_STARTUP)
        {
          GyroNeutral /= (END_STARTUP-SETTLE_STARTUP);
          GyroNeutral = (GyroNeutral+32)>>6; // Round to original accuracy
          min_gyro = max_gyro = 0;
          LastGyro = 0;
          GyroSum = 0;
        }
      }
    }
  }
}



/*******************************************************************************
* FUNCTION NAME: Gyro_Reset
* PURPOSE:       Reset gyrosum to input degrees(units = tenths of degree)
*******************************************************************************/
int Gyro_Reset(int degrees)
{
   GyroSum = degrees * DEG360 / 3600;
}



/*******************************************************************************
* FUNCTION NAME: Encoder_PID
* PURPOSE:       
* CALLED FROM:   various
*   gyro correction, IR correction, encoder correction, regular driving
*   Max free wheeling speed of encoders is ~2575 counts/sec or ~20.1 fps
*   Max under-load speed is ~13 fps
*******************************************************************************/
#define KP 31/10  // Basic proportional adjustment (31/10 works)
#define KI 15  // Cumulative error increases adjustment (15 works)
#define KD 0  // Sudden reaction to sudden changes
#define T  40 // This is executed at the slow loop rate of 40 Hz
#define CUMLIMIT 5000  // Limit to error accumulation (~ 2 complete robot 360's)
int Encoder_PID(long expected, long actual, int *LastActual, int *CumError)
{
  int p, i, d, pid, error;
  static unsigned long LastClock=0;

    /* Proportional */
    /* Add correction proportional to the current error */
    error = (int)(expected - actual);
    p = error * (int)KP;

    /* Integral */
    /* Correct accumulated error */
    i = *CumError * (int)KI / (int)T; // p covers current error

    /* Check for runaway accumulated error */
    *CumError += error; 
    if (*CumError > (int)CUMLIMIT) // Detect and avoid integral windup
      *CumError = (int)CUMLIMIT;
    if (*CumError < (int)-CUMLIMIT)// Detect and avoid integral windup
      *CumError = (int)-CUMLIMIT;

    /* Derivative */
    /* Sudden reaction to sudden changes in the error rate */
//    d = (error - *LastActual) * (int)KD;
//    *LastActual = error;

//  if (LastClock != Clockds)
  {
//    LastClock = Clockds;
#ifdef _DEBUG
    PrintString((char *) ", exp="); PrintSignedDecimal(expected);
    PrintString((char *) ", actual="); PrintSignedDecimal(actual);
    PrintString((char *) ", p="); PrintSignedDecimal(p);
    PrintString((char *) ", i="); PrintSignedDecimal(i);
    PrintString((char *) ", Cum="); PrintSignedDecimal(*CumError);
    PrintString((char *) ", d="); PrintSignedDecimal(d);
    PrintCR();
#endif
  }
//    pid = p;
    pid = p + i;
//    pid = p + i - d;
  return pid;
}


/*******************************************************************************
* FUNCTION NAME: Gyro_Steering
* PURPOSE:       
* CALLED FROM:   various
*   gyro correction, IR correction, encoder correction, regular driving
*   Max free wheeling speed of encoders is ~2575 counts/sec or ~20.1 fps
*   Max under-load speed is ~13 fps
*******************************************************************************/
#define SKP 40/10  // Basic proportional adjustment (40/10 works)
#define SKI 8  // Cumulative error increases adjustment (8 works)
#define SKD 0  // Sudden reaction to sudden changes (use 0)
#define ST  40 // This is executed at the slow loop rate of 40 Hz
#define SCUMLIMIT 5000  // Limit to error accumulation (~ 2 complete robot 360's)
int Gyro_Steering(int expected, int actual, int *LastActual, int *CumError)
{
  int p, i, d, pid, error;
  static unsigned long LastClock=0;

    /* Proportional */
    /* Add correction proportional to the current error */
    error = (expected - actual);
    p = error * (int)SKP;

    /* Integral */
    /* Correct accumulated error */
    i = *CumError * (int)SKI / (int)ST; // p covers current error

    /* Check for runaway accumulated error */
    *CumError += error; 
    if (*CumError > (int)SCUMLIMIT) // Detect and avoid integral windup
      *CumError = (int)SCUMLIMIT;
    if (*CumError < (int)-SCUMLIMIT)// Detect and avoid integral windup
      *CumError = (int)-SCUMLIMIT;

    /* Derivative */
    /* Sudden reaction to sudden changes in the error rate */
    d = (error - *LastActual) * (int)SKD;
    *LastActual = error;

//  if (LastClock != Clockds)
  {
    LastClock = Clockds;
#ifdef _DEBUG
    PrintString((char *) ", exp="); PrintSignedDecimal(expected);
    PrintString((char *) ", gyro="); PrintSignedDecimal(actual);
    PrintString((char *) ", p="); PrintSignedDecimal(p);
    PrintString((char *) ", i="); PrintSignedDecimal(i);
    PrintString((char *) ", Cum="); PrintSignedDecimal(*CumError);
    PrintString((char *) ", d="); PrintSignedDecimal(d);
    PrintCR();
#endif
  }
//    pid = p;
    pid = p + i;
//    pid = p + i - d;
  return pid;
}

//int Gyro_Degrees(void);

/*******************************************************************************
* FUNCTION NAME: Gyro_PID
* PURPOSE:       
* CALLED FROM:   user_routines_fast.c
*   gyro correction, IR correction, encoder correction, regular driving
*******************************************************************************/
#define GKP 6/10     // Basic proportional adjustment (8/10 works w/pads up at FULLSPEED)
//                                                      (6/10 w/pads down at HALFSPEED)
#define GKI 3         // Cumulative error increases adjustment (30 works w/pads up)
//                                                      (3 w/pads down at HALFSPEED)
#define GKD 0         // Sudden reaction to sudden changes (0)
#define GT  40        // This is executed at the slow loop rate of 40 Hz
#define GCUMLIMIT  127*GT/GKI // Limit to error accumulation
#define STARTMIN   8  // Number of cycles required to get out of the start motor stall
int Gyro_PID(int expected,char init)
{
  int p, i, d, pid, error;
  int actual;
  static int LastActual, CumError;
  static char Start, StartCount, Positive;

  actual = GYRO_DEGREES;

  if (init==TRUE)
  {
    CumError = 0;
    LastActual = actual;
    Start = TRUE;
    StartCount = 0;
    if ((actual - expected) < 0)
      Positive = FALSE;
    else
      Positive = TRUE;
  }
    if (StartCount > STARTMIN)
    {
//     if (Start)
//     {
//      PrintString((char *) " StartCount = "); PrintSignedDecimal(StartCount);
//     }
      Start = FALSE; // Start accumulating error only when near target setpoint
    }
    else
      StartCount++;

    /* Proportional */
    /* Add correction proportional to the current error */
    error = (actual - expected);
    p = error * (int)GKP;
#ifdef _DEBUG
    PrintString((char *) "p = ");
    PrintSignedDecimal(p);
#endif
    /* Integral */
    /* Correct accumulated error */
    /* Check for runaway accumulated error */
    if (error == 0) // If we hit target setpoint exactly then cut all power
      CumError = 0;
    if (Positive & (error < 0)) // If we passed the target then disgard accumulated error
    {
      CumError = 0;
      Positive = FALSE;
    }
    if (!Positive & (error > 0)) // If we passed the target then disgard accumulated error
    {
      CumError = 0;
      Positive = TRUE;
    }
    i = CumError * (int)GKI / (int)GT; // p covers current error

#ifdef _DEBUG
    PrintString((char *) ", Cum = "); PrintSignedDecimal(CumError);
#endif
    // Ignore initial stall condition before the robot begins to move
    // Don't accumulate unless we're stalled. This will then gradually overcome the stall
    if ((actual == LastActual) & !Start)
        CumError += error;

    if (CumError > (int)GCUMLIMIT) // Detect and avoid integral windup
      CumError = (int)GCUMLIMIT;
    if (CumError < (int)-GCUMLIMIT)// Detect and avoid integral windup
      CumError = (int)-GCUMLIMIT;

    /* Derivative */
    /* Sudden reaction to sudden changes in the error rate */
    d = (actual - LastActual) * (int)GKD;
    LastActual = actual;
#ifdef _DEBUG
    PrintString((char *) ", d="); PrintSignedDecimal(d);
#endif

//  pid = p;
  pid = p + i;
//  pid = p + i - d;
//  pid = p - d;

  return pid;
}


/*******************************************************************************
*	FUNCTION:		Initialize_Interrupts()
*	PURPOSE:		Initializes the interrupt hardware.
*	CALLED FROM:	user_routines.c/User_Initialization()
*******************************************************************************/
void Initialize_Interrupts(void)  
{
	// initialize external interrupt 1 (INT2 on user 18F8520)
	TRISBbits.TRISB2 = 1;		// make sure the RB2/INT2 pin is configured as an input [108]
								//
	INTCON3bits.INT2IP = 0;		// 0: interrupt 1 is low priority (leave at 0 for IFI controllers) [91]
								// 1: interrupt 1 is high priority
								//
	INTCON2bits.INTEDG2 = 1;	// 0: trigger on the falling-edge [90]
								// 1: trigger on the rising-edge
								//
	INTCON3bits.INT2IE = 1;		// 0: disable interrupt	1 [91]
								// 1: enable interrupt 1
	
	// initialize external interrupt 2 (INT3 on user 18F8520)
	TRISBbits.TRISB3 = 1;		// make sure the RB3/CCP2/INT3 pin is configured as an input [108]
								//
	INTCON2bits.INT3IP = 0;		// 0: interrupt 2 is low priority (leave at 0 for IFI controllers) [90]
								// 1: interrupt 2 is high priority
								//
	INTCON2bits.INTEDG3 = 1;	// 0: trigger on the falling-edge [90]
								// 1: trigger on the rising-edge
								//
	INTCON3bits.INT3IE = 1;		// 0: disable interrupt	2 [91]
								// 1: enable interrupt 2
}
