/*******************************************************************************
* FILE NAME: user_routines.c
*
* DESCRIPTION:
*  This file contains the default mappings of inputs  
*  (like switches, joysticks, and buttons) to outputs on the RC.  
*
* USAGE:
*  You can either modify this file to fit your needs, or remove it from your 
*  project and replace it with a modified copy. 
*  
* Notes: digital input pins are "high" or 1 when open and "low" or 0 when closed
*        OI inputs are TRUE or "1" when the button is pressed
*
* SPECIAL NOTE:
*  Compiler flags used in this project are:
*  _FRC_BOARD compiles the FRC version of certain routine calls
*  _NO_PRINTS disables all serial output print statements used
*  _SIMULATOR disables non-MPLAB SIM compliant code
*  _DIAGNOSTICS_ enables some diagnostic routines, otherwise disabled to save on space
*
*******************************************************************************/

#ifdef _DEBUG
#include "print_routines.h"
#endif

#include "ifi_aliases.h"
#include "ifi_default.h"
#include "ifi_utilities.h"

#include "timers.h"

#include "clock_routines.h"
#include "sensor_routines.h"
#include "math.h"
#define VARIABLE_DECLARE  /* causes certain global variables to be declared only once */
  #include "user_routines.h"
  #include "auto_routines.h"
  #include "motor_routines.h"
#undef VARIABLE_DECLARE

/*******************************************************************************
* FUNCTION NAME: User_Initialization
* PURPOSE:       This routine is called first (and only once) in the Main function.  
*                You may modify and add to this function.
* CALLED FROM:   main.c
* ARGUMENTS:     none
* RETURNS:       void
*******************************************************************************/
void User_Initialization (void)
{

  Set_Number_of_Analog_Channels(SIXTEEN_ANALOG);    /* DO NOT CHANGE! */

/* FIRST: Set up the I/O pins you want to use as digital INPUTS. */
  digital_io_01 = digital_io_02 = digital_io_03 = digital_io_04 = INPUT;
  digital_io_05 = digital_io_06 = digital_io_07 = digital_io_08 = INPUT;
  digital_io_09 = digital_io_10 = digital_io_11 = digital_io_12 = INPUT;
  digital_io_13 = digital_io_14 = digital_io_15 = digital_io_16 = INPUT;
  digital_io_17 = INPUT;  digital_io_18 = INPUT;  /* Used for pneumatic pressure switch. */

/* SECOND: Set up the I/O pins you want to use as digital OUTPUTS. */

/* THIRD: Initialize the values on the digital outputs. */

/* FOURTH: Set your initial PWM values.  Neutral is 127. */
  pwm01 = pwm02 = pwm03 = pwm04 = pwm05 = pwm06 = pwm07 = pwm08 = 127;
  pwm09 = pwm10 = pwm11 = pwm12 = pwm13 = pwm14 = pwm15 = pwm16 = 127;

/* FIFTH: Set your PWM output types for PWM OUTPUTS 13-16.
  /*   Choose from these parameters for PWM 13-16 respectively:               */
  /*     IFI_PWM  - Standard IFI PWM output generated with Generate_Pwms(...) */
  /*     USER_CCP - User can use PWM pin as digital I/O or CCP pin.           */

  Setup_PWM_Output_Type(IFI_PWM,IFI_PWM,IFI_PWM,IFI_PWM);

  /* 
     Example: The following would generate a 40KHz PWM with a 50% duty cycle on the CCP2 pin:

         CCP2CON = 0x3C;
         PR2 = 0xF9;
         CCPR2L = 0x7F;
         T2CON = 0;
         T2CONbits.TMR2ON = 1;

         Setup_PWM_Output_Type(USER_CCP,IFI_PWM,IFI_PWM,IFI_PWM);
  */

  /* Add any other initialization code here. */
  /* Initialize relay outputs */
    relay1_fwd = 0; relay1_rev = 0;
    relay2_fwd = 0; relay2_rev = 0;
    relay3_fwd = 0; relay3_rev = 0;
    relay4_fwd = 0; relay4_rev = 0;
    relay5_fwd = 0; relay5_rev = 0;
    relay6_fwd = 0; relay6_rev = 0; // Push-down pads
    relay7_fwd = 0; relay7_rev = 0; // Tetra Grabber
    relay8_fwd = 0; relay8_rev = 0; // Compressor

  Initialize_Clock();

  Initialize_Serial_Comms();   //Use no "print" statement before the serial comms initialize
  
  Initialize_Interrupts();

  Putdata(&txdata);             /* DO NOT CHANGE! */

  User_Proc_Is_Ready();         /* DO NOT CHANGE! - last line of User_Initialization */
}

/*******************************************************************************
* FUNCTION NAME: Process_Data_From_Master_uP
* PURPOSE:       Executes every 26.2ms when it gets new data from the master 
*                microprocessor.
* CALLED FROM:   main.c
* ARGUMENTS:     none
* RETURNS:       void
*******************************************************************************/
void Process_Data_From_Master_uP(void)
{
  pwm01 = pwm02 = pwm03 = pwm04 = pwm05 = pwm06 = pwm07 = pwm08 = 127;
  pwm09 = pwm10 = pwm11 = pwm12 = pwm13 = pwm14 = pwm15 = pwm16 = 127;

}

/*******************************************************************************
* FUNCTION NAME: User_Driver
* PURPOSE:       Performs the default mappings of inputs to outputs for the
*                Robot Controller.
* CALLED FROM:   this file, Process_Data_From_Master_uP routine
* ARGUMENTS:     none
* RETURNS:       void
*******************************************************************************/
#define HOLDELAY 5  //Delay in tenths of seconds to allow slowdown to a stop
void User_Driver(void)
{
  static char TetraToggle=0, PadsToggle=0;
  static char counter = 0;
  static unsigned long LastClock=0, LastPrint=0;
  unsigned int telescope_pot;
  int arm_pot;
  static char arm_still=FALSE;


  if (LastPrint < Clocksec)
  {
    LastPrint = Clocksec;
#ifdef _DEBUG
    PrintString((char *)"LEFT = ");
    PrintSignedDecimal(Left_Encoder_Count);
    PrintString((char *)",RIGHT = ");
    PrintSignedDecimal(Right_Encoder_Count);
//    PrintString(",WHL_R1 = ");
//    PrintSignedDecimal(WHL_R1);
//    PrintString(",WHL_L1 = ");
//    PrintSignedDecimal(WHL_L1);
#endif
  }

  if (!OISWITCH1) // Selectable by OI switch */
  {
    /*---------- 1 Joystick Drive ----------------------------------------------
     *--------------------------------------------------------------------------
     */  

     Smooth_Joystick(&JOY_DRIVE1A);
     Smooth_Joystick(&JOY_DRIVE1B);

     //Continuous_Deadband(&JOY_DRIVE1A);
     //Continuous_Deadband(&JOY_DRIVE1B);
     WHL_R1 = WHL_R2 = -Limit_Mix(JOY_DRIVE1A + JOY_DRIVE1B);
     WHL_L1 = WHL_L2 = -Limit_Mix(JOY_DRIVE1A - JOY_DRIVE1B);

     JOY_DRIVE2 = 0;
  }
  else
  {
   /*---------- 2 Joystick Drive ----------------------------------------------
    *--------------------------------------------------------------------------
    */

    Smooth_Joystick(&JOY_DRIVE2);
    Smooth_Joystick(&JOY_DRIVE1A);

  //  Continuous_Deadband(&JOY_DRIVE2);
  //  Continuous_Deadband(&JOY_DRIVE1A);

    WHL_R1 = WHL_R2 = -JOY_DRIVE1A;
    WHL_L1 = WHL_L2 = -JOY_DRIVE2;
    JOY_DRIVE1B = 0;
  }

 /*---------- Hold Stationary Position --------------------------------------
  *--------------------------------------------------------------------------
  */
//#ifdef MLM  /**************** DEBUG ****************/
    if ((JOY_DRIVE1A == 0) & (JOY_DRIVE1B == 0) & (JOY_DRIVE2 == 0))
    {
//      PrintString((char *) " PadsCounter =");
//      PrintDecimal(PadsCounter);
//      PrintCR();

      if (notmoving)
      {
        if (LastClock == Clockds) // Force robot to "settle" after what may be a sudden stop
        {
          LastClock = 0;
          /* Disable interrupts while we zero the shaft encoders */
          INTCONbits.GIEL = 0;
          Right_Encoder_Count = 0;
          Left_Encoder_Count = 0;
          INTCONbits.GIEL = 1;
        }
        Hold_Position((int)FALSE, (int)0, (int)0);
      }
      else
      {
        /* Disable interrupts while we zero the shaft encoders */
        INTCONbits.GIEL = 0;
        Right_Encoder_Count = 0;
        Left_Encoder_Count = 0;
        INTCONbits.GIEL = 1;

        LastClock = Clockds + HOLDELAY;
        Hold_Position((int)TRUE, (int)0, (int)0);
        notmoving = TRUE;
//        ++PadsCounter;
      }
    }
    else
    {
      notmoving = FALSE;
    }
//#endif /**************** DEBUG ****************/

 /*---------- Arm -----------------------------------------------------------
  *--------------------------------------------------------------------------
  */

/* Arm at highest extension reaches 150.5" (12.5 feet) */

/*  Loading station low = arm_pot 726 */
/*  Loading station high = arm_pot 737 */
/*  Take tetra off loading station - arm pot 751 */
/*  Arm raised above 5' goal - arm pot 751 */
/*  Home position - arm pot ARMLIMITLOW */
/*  Capping 5' height, 1st tetra - arm pot = 768 */
/*  Capping 7' height, 1st tetra - arm pot = 779 */
/* Floor pickup - arm pot = 724 */

/* about 335 degrees of travel on the pot */
/* speculate there are ~1.5 degrees per tick */

//#define ARMLIMITLOW 1023 
//#define ARMLIMITHIGH 0
//#define ARMLIMITLOW 60   //Regular pot
//#define ARMLIMITHIGH 995 // Regular pot
//#define ARMLIMITLOW 913 //MLM  SBPLI 910 works
//#define ARMLIMITHIGH 820 //MLM  SBPLI
//#define ARMLIMITLOW 916 //MLM  Philly 907 bottom
//#define ARMLIMITHIGH 815 //MLM  Philly 810 top
//#define ARMLIMITLOW 924 //MLM  Philly 918 bottom was 920
//#define ARMLIMITHIGH 826 //MLM  Philly 826 top
#define ARMLIMITLOW 933
#define ARMLIMITHIGH 833


  Arm_Deadband(&ARM_INPUT); // Based on joystick position, assumes 0 is neutral
  ARM_INPUT += ARMNEUTRAL; // In case neutral is not zero
//  ARM_PWM = ARM_INPUT;

  // Check for pwm limits
  if (ARM_INPUT > 127) ARM_INPUT = 127;
  if (ARM_INPUT < -127) ARM_INPUT = -127;

//  ArmJoint(ARM_INPUT);
   /* negative is up, positive is down */
   arm_pot = (int) Get_Analog_Value(ARM_POT);

//#ifdef MLM
   if (ARM_INPUT == ARMNEUTRAL) // Arm is supposed to be stationary-neutral may not be 0
   {
     if (arm_still == FALSE)
     { // Initialize arm hold
        arm_still = TRUE;
        Hold_Arm_Position(TRUE, arm_pot, &ARM_INPUT);
     }
     else
     { // Keep on holding the arm still
        Hold_Arm_Position(FALSE, arm_pot, &ARM_INPUT);  
     }
   }
   else
     arm_still = FALSE; // arm is in motion

   /**** Limit check and override ****/
   if (
     ( (ARM_INPUT > 0) && (arm_pot < ARMLIMITLOW) ) ||
     ( (ARM_INPUT < 0) && (arm_pot > ARMLIMITHIGH) )
      ) //down or up
   {
       ARM_PWM = ARM_INPUT;
   }  
   else
       ARM_PWM = ARMNEUTRAL;
//#endif

#ifdef _DEBUG
//  PrintString((char *) "ARM_INPUT= ");
//  PrintSignedDecimal(ARM_INPUT);
//  PrintCR();
  PrintString((char *) "ARM_POT= ");
  PrintDecimal(arm_pot);
//  PrintString((char *) "ARM_PWM= ");
//  PrintSignedDecimal(ARM_PWM);
//  PrintCR();
#endif


//  if (!OISWITCH2) // Override all limits from the OI
//  {
//  }

 /*-------------- Telescoping arm -------------------------------------------
  *--------------------------------------------------------------------------
  */
/*  Capping 1st tetra 5' goal - tele pot = 576 */
/*  Capping 1st tetra 7' goal - tele pot =  */
/*  Loading station - tele pot = TELELIMITIN */
/* each pot tick = .08" */

//    ArmTelescope(ip2_aux);
//#define TELELIMITOUT 825   // arm out reaches 111" (max is 856, cut back for motor lag)
//#define TELELIMITIN 270    // arm in reaches 59"
//#define TELELIMITOUT 600  //600 arm out reaches 111" (max is ?, cut back for motor lag)
//#define TELELIMITIN 50    //50 arm in reaches 59"
#define TELELIMITOUT 796   // arm out reaches 111" (max is 836) (coasts to 820)
#define TELELIMITIN 279    // arm in reaches 59" (min is 249)
//#define TELELIMITOUT 1023  // No limits
//#define TELELIMITIN 0    // No limits

    telescope_pot = Get_Analog_Value(TELESCOPE_POT);

//    if (TELESCOPE_OUT_INPUT)
    if (TELESCOPE_OUT_INPUT  && (telescope_pot < TELELIMITOUT))
    {
      TELESCOPE_PWM1 = 127;
//      TELESCOPE_PWM1 = 100;  //100, 49 too slow
      PrintString((char *) " TELE out ");
    }
//    else if (TELESCOPE_IN_INPUT)
    else if (TELESCOPE_IN_INPUT && (telescope_pot > TELELIMITIN))
    {
      PrintString((char *) " TELE in ");
//      TELESCOPE_PWM1 = -100;  //100;
      TELESCOPE_PWM1 = -127;
    }
    else
    {
      TELESCOPE_PWM1 = 0; // stop all engines
    }
#ifdef _DEBUG
  PrintString((char *) ",  TELE_POT= ");
  PrintDecimal(telescope_pot);
//  PrintString((char *) " TELE_OUT= ");
//  PrintDecimal(TELESCOPE_OUT_INPUT);
//  PrintString((char *) ",  TELE_IN= ");
//  PrintDecimal(TELESCOPE_IN_INPUT);
//  PrintString((char *) ",  TELE_PWM= ");
//  PrintDecimal(TELESCOPE_PWM1);
  PrintCR();
#endif

 /*------------- Solenoids --------------------------------------------------
  *--------------------------------------------------------------------------
  */

  /** Tetra grabber **/
  if (!TETRA_GRABBER_INPUT) //P1 trigger
  {
    TetraToggle = 1;
  }
  if (TETRA_GRABBER_INPUT && TetraToggle)
  {
    TETRA_GRABBER = !TETRA_GRABBER;  //solenoid port 4
    TETRA_GRABBER_REV = 0; //solenoid port 2
    TetraToggle = 0;
    counter = 0;
    PrintString((char *) " ***tetra trigger ");
    PrintCR();
  }

  /** Push-down pads (gets tires 1.25" above carpet) **/
//  if (!PUSH_DOWN_PADS_INPUT)  // Change on button press
//  {
//    PadsToggle = 1;
//  }
//  if (!PUSH_DOWN_PADS_INPUT && PadsToggle)  // Change on button release
//  if (PUSH_DOWN_PADS_INPUT && PadsToggle) // Change on button press
  if (PUSH_DOWN_PADS_INPUT) // Down while button is held
  {
//    PUSH_DOWN_PADS = !PUSH_DOWN_PADS;
    PUSH_DOWN_PADS = 1;
    PUSH_DOWN_PADS_REV = 0;
//    PadsToggle = 0;
    PadsCounter = 0; // To suspend Hold_Position while we are airborne
//    notmoving = FALSE;
  }
  else
  {
    PUSH_DOWN_PADS = 0;
    PUSH_DOWN_PADS_REV = 0;
  }
    
   
} /* END Default_Routine(); */

/******************************************************************************/
/******************************************************************************/
/******************************************************************************/
