/*******************************************************************************
* FILE NAME: user_routines_fast.c
*
* DESCRIPTION:
*  Interrupts are handled here, as well as anything requiring fast loop processing
*
*******************************************************************************/

#include <usart.h>
#include <adc.h>
#include <capture.h>
#include <timers.h>

#ifdef _DEBUG
#include "print_routines.h"
#endif

#include "ifi_aliases.h"
#include "ifi_default.h"
#include "ifi_utilities.h"

#include "clock_routines.h"
#include "auto_routines.h"
#include "user_routines.h"
#include "motor_routines.h"
#include "sensor_routines.h"


/*******************************************************************************
* FUNCTION NAME: InterruptVectorLow
* PURPOSE:       Low priority interrupt vector
* CALLED FROM:   nowhere by default
* ARGUMENTS:     none
* RETURNS:       void
* DO NOT MODIFY OR DELETE THIS FUNCTION 
*******************************************************************************/
#pragma code InterruptVectorLow = LOW_INT_VECTOR
void InterruptVectorLow (void)
{
  _asm
    goto InterruptHandlerLow  /*jump to interrupt routine*/
  _endasm
}


/*******************************************************************************
* FUNCTION NAME: InterruptHandlerLow
* PURPOSE:       Low priority interrupt handler
* If you want to use these external low priority interrupts or any of the
* peripheral interrupts then you must enable them in your initialization
* routine.  There is great potential for glitchy code if good
* interrupt programming practices are not followed.  Especially read p. 28 of
* the "MPLAB(R) C18 C Compiler User's Guide" for information on context saving.
* CALLED FROM:   this file, InterruptVectorLow routine
* ARGUMENTS:     none
* RETURNS:       void
*******************************************************************************/
#pragma code
#pragma interruptlow InterruptHandlerLow save=PROD,section("MATH_DATA"),section(".tmpdata")

void InterruptHandlerLow ()     
{                               
  unsigned char Port_B;
  unsigned char Port_B_Delta;       
  static unsigned char Old_Port_B = 0xFF; // state of port b the last time
											// this function was called  
  static unsigned char	t100ms=0;
  static unsigned int	  t1sec=0;

  /*** Service Internal timer interrupts ***/

	if (INTCONbits.TMR0IF) // timer 0 Interrupt
	{
		INTCONbits.TMR0IF = 0; // clear the interrupt flag
	}

	else if (PIR1bits.TMR1IF) // timer 1 interrupt - used by the IR receiver
	{
		PIR1bits.TMR1IF = 0; // clear the interrupt flag
	} 
 
  else if (PIR3bits.TMR4IF)	/* TIMER 4 INTERRUPT */
  {
    /** This provides us with a clock for timing events **/

    PIR3bits.TMR4IF = 0;	/* Clear Timer interrupt flag */
    WriteTimer4(6);	  	/* Reset Timer to overflow at 4ms */
    Clockms += 4; /* milliseconds */
    GyroTicks++;  // Update the gyro readings
    if (++t100ms >= 25)
    {
      t100ms = 0;
      Clockds++; /* tenths of seconds */
      if (++t1sec >= 10)
      {
        t1sec = 0;
        Clocksec++; /* seconds */
      }
    }
  }                             

  /*** Service External Interrupts ***/

	else if (INTCON3bits.INT2IF) // external interrupt 1
	{ 
		INTCON3bits.INT2IF = 0; // clear the interrupt flag
	    // if used, call the interrupt 1 handler here
      /* Could double encoder resolution by also putting "B" on an interrupt */
      /** One encoder revolution (256 interrupts) = 16.28" of travel **/
      /*  One encoder "tick" = .06" */
		if(rc_dig_in07)  //Quad encoder B signal
        {
			Left_Encoder_Count--;
            Total_Left_Encoder_Count--;
        }
		else
        {
			Left_Encoder_Count++;
            Total_Left_Encoder_Count++;
        }

        /* Make sure the other encoder hasn't also fired */
        if (INTCON3bits.INT3IF == 1)
        {
            INTCON3bits.INT3IF = 0; // clear the interrupt flag
 		  	if(rc_dig_in08)  //Quad encoder B signal
            {
		  		Right_Encoder_Count++;
		    	Total_Right_Encoder_Count++;
            }
		  	else
            {
		  		Right_Encoder_Count--;
			    Total_Right_Encoder_Count--;
            }
        }
	}

	else if (INTCON3bits.INT3IF) // external interrupt 2 
	{
		INTCON3bits.INT3IF = 0; // clear the interrupt flag
		// if used, call the interrupt 2 handler here
		if(rc_dig_in08)  //Quad encoder B signal
        {
			Right_Encoder_Count++;
			Total_Right_Encoder_Count++;
        }
		else
        {
			Right_Encoder_Count--;
			Total_Right_Encoder_Count--;
        }

        /* Make sure the other encoder hasn't also fired */
        if (INTCON3bits.INT2IF == 1)
        {
		  INTCON3bits.INT2IF = 0; // clear the interrupt flag
			if(rc_dig_in07)  //Quad encoder B signal
            {
				Left_Encoder_Count--;
                Total_Left_Encoder_Count--;
            }
			else
            {
				Left_Encoder_Count++;
                Total_Left_Encoder_Count++;
            }
        }
	}

	else if (INTCONbits.RBIF) // external interrupts 3 through 6
	{
		Port_B = PORTB; // remove the "mismatch condition" by reading port b            
		INTCONbits.RBIF = 0; // clear the interrupt flag
		Port_B_Delta = Port_B ^ Old_Port_B; // determine which bits have changed
		Old_Port_B = Port_B; // save a copy of port b for next time around
	 
		if(Port_B_Delta & 0x10) // did external interrupt 3 change state? - IR sensor 1
		{
//			Int_3_Handler(Port_B & 0x10 ? 1 : 0); // call the interrupt 3 handler (in receiver.c)
		}
		if(Port_B_Delta & 0x20) // did external interrupt 4 change state? - IR sensor 2
		{
//			Int_4_Handler(Port_B & 0x20 ? 1 : 0); // call the interrupt 4 handler (in receiver.c)
		}
		if(Port_B_Delta & 0x40) // did external interrupt 5 change state? - IR sensor 3
		{
//			Int_5_Handler(Port_B & 0x40 ? 1 : 0); // call the interrupt 5 handler (in receiver.c)
		}
		if(Port_B_Delta & 0x80) // did external interrupt 6 change state? - IR sensor 4
		{
//			Int_6_Handler(Port_B & 0x80 ? 1 : 0); // call the interrupt 6 handler (in receiver.c)
		}
	}
}


/*******************************************************************************
* FUNCTION NAME: Process_Data_From_Local_IO
* PURPOSE:       Execute user's realtime code.
* You should modify this routine by adding code which you wish to run fast.
* It will be executed every program loop, and not wait for fresh data 
* from the Operator Interface.
* CALLED FROM:   main.c
* ARGUMENTS:     none
* RETURNS:       void
*******************************************************************************/
void Process_Data_From_Local_IO(void)
{
  static long LastClock=0, LastCount=-127;

  Gyro(); // Update gyro heading

}

/******************************************************************************/
/******************************************************************************/
/******************************************************************************/
